//
//  FAConfiguration.h
//  FollowAnalytics
//
//  Created by Adrian Tofan on 04/05/2018.
//  Copyright © 2018 FOLLOW APPS. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <FollowAnalytics/FollowAnalyticsTypes.h>

@class FollowAnalyticsMutableConfiguration;

/**
 Development time defaults used by FollowAnalytics.
 */
@interface FollowAnalyticsConfiguration : NSObject <NSCopying, NSMutableCopying>

/**
 The default state of optInAnalytics used if the user did not express his choiche. Set to TRUE.
 */
@property (nonatomic, readonly, assign) BOOL optInAnalyticsDefault;

/**
 The API key coresponding to this apps bundle identifier as defined on the FollowAnlaytics.com backoffice. Default
 NULL, must be set before use.
 */
@property (nonatomic, readonly, strong, nonnull) NSString* apiKey;

/**
 The app group shared with the notification extension if any (needed for the badge management). Default null;
 */
@property (nonatomic, readonly, strong, nullable) NSString* appGroup;

/**
 Set to true during development in order to have more verbose log output and on screen diagnostic display. Default false.

 Can also be overriden at runtime by setting FOLLOW_ANALYTICS_DEBUG_MODE environment variable to true or false.
 */
@property (nonatomic, readonly, assign) BOOL debug;

/**
 All API calls are made in one of the two modes:

 - FolloAnalyticsAPIModeProd: the server stores the logs and agregates them in the reports

 - FolloAnalyticsAPIModeDev: the server responds to calls in a relevant way, without storing call data. Used during development in order to not poluate production app data with development statistics.

 It defaults to:

 - on the simulator is FolloAnalyticsAPIModeDev

 - when debug is on : FolloAnalyticsAPIModeDev

 - otherwise FolloAnalyticsAPIModeProd

 Can also be overriden at runtime by setting FOLLOW_ANALYTICS_API_MODE environment variable to the value of on of FolloAnalyticsAPIModeProd or FolloAnalyticsAPIModeDev.
 */
@property (nonatomic, readonly, assign) FollowAnalyticsAPIMode apiMode;

/**
 Enable and disable crash reporting. Default true.
 */
@property (nonatomic, readonly, assign) BOOL crashReportingEnabled;

/**
 Current api environment "" = prod. For the time being, the obsolete (since version 5.4) [FAFollowApps setEnvironment:], if used, takes precedence over FolloAnalyticsConfiguration
 */
@property (nullable, nonatomic, readonly, strong) NSString* environment;

/**
 Creates and returns a instance modified inside of the configurationBlock. It builds a mutable
 instance, passes that instance to the configuration block which can modify it as it needs.
 Finnaly returns a nonmutable copy of the mutable instance.

 @warning: runtime variable overrides are still honored.

 @code
  FollowAnalyticsConfiguration *c = [FollowAnalyticsConfiguration configurationWith:^(FollowAnalyticsMutableConfiguration * _Nonnull c) {
    c.optInAnalyticsDefault = false;
  }];
 @endcode

 @param configurationBlock convenience configuration block called with a FollowAnalyticsMutableConfiguration
 @return a (imutable) instance of FollowAnalyticsConfiguration
 */
+(nonnull instancetype) configurationWith:(nonnull void (^)(FollowAnalyticsMutableConfiguration* _Nonnull  c))  configurationBlock;

@end
